package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.AsymmetricCipherKeyPair;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.generators.ECKeyPairGenerator;
import cn.com.infosec.crypto.params.ECDomainParameters;
import cn.com.infosec.crypto.params.ECKeyGenerationParameters;
import cn.com.infosec.crypto.params.ECKeyParameters;
import cn.com.infosec.crypto.params.ECPrivateKeyParameters;
import cn.com.infosec.crypto.params.ECPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.math.ec.ECAlgorithms;
import cn.com.infosec.math.ec.ECConstants;
import java.math.BigInteger;
import java.security.SecureRandom;

public class ECNRSigner implements DSA {
   private boolean forSigning;
   private ECKeyParameters key;
   private SecureRandom random;

   public void init(boolean forSigning, CipherParameters param) {
      this.forSigning = forSigning;
      if (forSigning) {
         if (param instanceof ParametersWithRandom) {
            ECNRSigner var10000 = this;
            ParametersWithRandom var3;
            this.random = (var3 = (ParametersWithRandom)param).getRandom();
            var10000.key = (ECPrivateKeyParameters)var3.getParameters();
         } else {
            this.random = new SecureRandom();
            this.key = (ECPrivateKeyParameters)param;
         }
      } else {
         this.key = (ECPublicKeyParameters)param;
      }

   }

   public BigInteger[] generateSignature(byte[] digest) {
      if (!this.forSigning) {
         throw new IllegalStateException("not initialised for signing");
      } else {
         BigInteger var2;
         int var10000 = (var2 = ((ECPrivateKeyParameters)this.key).getParameters().getN()).bitLength();
         BigInteger var3;
         BigInteger var10001 = var3 = new BigInteger;
         var10001.<init>(1, digest);
         int var10002 = var10001.bitLength();
         ECPrivateKeyParameters digest1 = (ECPrivateKeyParameters)this.key;
         if (var10002 > var10000) {
            throw new DataLengthException("input too large for ECNR key.");
         } else {
            BigInteger var5;
            AsymmetricCipherKeyPair var8;
            ECKeyPairGenerator var9;
            do {
               var9 = new ECKeyPairGenerator();
               ECDomainParameters var4 = digest1.getParameters();
               var9.init(new ECKeyGenerationParameters(var4, this.random));
            } while((var5 = ((ECPublicKeyParameters)(var8 = var9.generateKeyPair()).getPublic()).getQ().getX().toBigInteger().add(var3).mod(var2)).equals(ECConstants.ZERO));

            BigInteger var6 = digest1.getD();
            var6 = ((ECPrivateKeyParameters)var8.getPrivate()).getD().subtract(var5.multiply(var6)).mod(var2);
            return new BigInteger[]{var5, var6};
         }
      }
   }

   public boolean verifySignature(byte[] digest, BigInteger r, BigInteger s) {
      if (!this.forSigning) {
         BigInteger var4;
         ECPublicKeyParameters var6;
         int var10000 = (var4 = (var6 = (ECPublicKeyParameters)this.key).getParameters().getN()).bitLength();
         BigInteger var5;
         BigInteger var10001 = var5 = new BigInteger;
         var10001.<init>(1, digest);
         if (var10001.bitLength() <= var10000) {
            if (r.compareTo(ECConstants.ONE) >= 0 && r.compareTo(var4) < 0) {
               return s.compareTo(ECConstants.ZERO) >= 0 && s.compareTo(var4) < 0 ? r.subtract(ECAlgorithms.sumOfTwoMultiplies(var6.getParameters().getG(), s, var6.getQ(), r).getX().toBigInteger()).mod(var4).equals(var5) : false;
            } else {
               return false;
            }
         } else {
            throw new DataLengthException("input too large for ECNR key.");
         }
      } else {
         throw new IllegalStateException("not initialised for verifying");
      }
   }
}
